@@ -60,6 +60,7 @@ struct _MicrofeedMain {
 	int polling;
 	int wakeup_pipe[2];
 	int exit_requested : 1;
+	int poll_fds_valid : 1;
 };
 
 static void dispatch_status_changed(DBusConnection* connection, DBusDispatchStatus new_status, void* data);
@@ -119,11 +120,6 @@ MicrofeedMain* microfeed_main_new_with_dbus_connection(DBusConnection* connectio
 	microfeed_main = microfeed_memory_allocate(MicrofeedMain);
 	microfeed_main->mutex = microfeed_mutex_new();
 	microfeed_main->connection = connection;
-	microfeed_main->timeouts = NULL;
-	microfeed_main->watches = NULL;
-	microfeed_main->watches_count = 0;
-	microfeed_main->poll_fds = NULL;
-	microfeed_main->polling = 0;
 	if (pipe(microfeed_main->wakeup_pipe)) {
 		fprintf(stderr, "ERROR: Pipe failed.\n");
 		
@@ -226,7 +222,10 @@ void microfeed_main_loop(MicrofeedMain* microfeed_main) {
 			milliseconds = -1;
 		}
 		
-		if (!microfeed_main->poll_fds) {
+		if (!microfeed_main->poll_fds_valid) {
+			if (microfeed_main->poll_fds) {
+				microfeed_memory_free(microfeed_main->poll_fds);
+			}
 			microfeed_main->poll_fds = pollfd = microfeed_memory_allocate_bytes((microfeed_main->watches_count + 1) * sizeof(struct pollfd));
 			pollfd->fd = microfeed_main->wakeup_pipe[0];
 			pollfd->events = POLLIN;
@@ -241,6 +240,7 @@ void microfeed_main_loop(MicrofeedMain* microfeed_main) {
 					pollfd->events = POLLIN;
 				}
 			}
+			microfeed_main->poll_fds_valid = 1;
 		}
 
 		if (microfeed_main->exit_requested) {
@@ -276,7 +276,7 @@ void microfeed_main_loop(MicrofeedMain* microfeed_main) {
 			dummy = read(microfeed_main->wakeup_pipe[0], buffer, 1024);
 		}
 		
-		if (microfeed_main->poll_fds) {
+		if (microfeed_main->poll_fds_valid) {
 			for (watch = microfeed_main->watches; watch; watch = microfeed_main->watch_iterator) {
 				microfeed_main->watch_iterator = watch->next;
 				if ((pollfd = watch->pollfd)) {
@@ -413,8 +413,7 @@ MicrofeedWatch* microfeed_main_add_watch(MicrofeedMain* microfeed_main, int fd,
 	microfeed_main->watches = watch;
 	microfeed_main->watches_count++;
 
-	microfeed_memory_free(microfeed_main->poll_fds);
-	microfeed_main->poll_fds = NULL;
+	microfeed_main->poll_fds_valid = 0;
 	wakeup_poll(microfeed_main);
 
 	microfeed_mutex_unlock(microfeed_main->mutex);
@@ -463,8 +462,7 @@ void microfeed_main_remove_watch(MicrofeedMain* microfeed_main, MicrofeedWatch*
 		microfeed_memory_free(watch);	
 		microfeed_main->watches_count--;
 	
-		microfeed_memory_free(microfeed_main->poll_fds);
-		microfeed_main->poll_fds = NULL;
+		microfeed_main->poll_fds_valid = 0;
 		wakeup_poll(microfeed_main);
 	}
 
