@@ -28,26 +28,31 @@
 #include <stdio.h>
 #include <stdint.h>
 
+#define METADATA_LAST_UPDATED "last_updated"
+
+typedef struct _ItemDataProperty ItemDataProperty;
+
 struct _MicrofeedFeed {
-	unsigned int reference_count;
-	MicrofeedWeakReference* weak_reference;
+	MicrofeedObject object;
 
 	char* uri;
 	char* name;
+	MicrofeedPublisher* publisher;
+	char* object_path;
 	MicrofeedFeedPermission feed_permission;
 	MicrofeedItem* metadata_item;
-	MicrofeedPublisher* publisher;
 	MicrofeedFeedCallbacks callbacks;
 	void* user_data;
 	MicrofeedFeedIterator* iterators;
 	int subscriber_count;
+	int updating;
 	MicrofeedDatabase* database;
 	MicrofeedDatabaseIndex* database_index;
 	MicrofeedDatabase* statuses_database;
+	MicrofeedDatabase* metadata_database;
 	MicrofeedDatabase* item_data_database;
-	MicrofeedMutex* mutex;
-	int updating;
 	MicrofeedStore* get_item_data_mutexes;
+	MicrofeedStore* item_data_properties;
 };
 
 struct _MicrofeedFeedIterator {
@@ -66,23 +71,34 @@ typedef struct {
 	MicrofeedMutex* mutex;
 } GetItemDataMutex;
 
+struct _ItemDataProperty {
+	char* property_name;
+	MicrofeedFeed* item_data_feed;
+};
+
+static void free_function(MicrofeedFeed* feed);
+static const char* item_data_property_get_property_name(ItemDataProperty* item_data_property);
+static void handle_item_property_change(MicrofeedFeed* feed, MicrofeedItem* old_item, MicrofeedItem* new_item);
 static void index_function(void* key, const size_t key_size, void* data, const size_t data_size, void** index_key, size_t* index_key_size);
 static int compare_function(const void* key1, const size_t key1_size, const void* key2, const size_t key2_size);
 static void remove_old_items(MicrofeedFeed* feed);
 static const char* get_item_data_mutex_get_uid(GetItemDataMutex* get_item_data_mutex);
 
+static MicrofeedClass microfeed_feed_class = {
+	"MicrofeedFeed",
+	(MicrofeedObjectFreeFunction)free_function
+};
+
 MicrofeedFeed* microfeed_feed_new(MicrofeedPublisher* publisher, const char* uri, const char* name, MicrofeedFeedPermission feed_permission, MicrofeedFeedCallbacks* callbacks, void* user_data) {
 	MicrofeedFeed* feed;
 	char* database_name;
 	char* s;
 	
-	feed = (MicrofeedFeed*)microfeed_memory_allocate(MicrofeedFeed);
-
-	feed->reference_count = 1;
-	feed->weak_reference = NULL;
-
+	feed = microfeed_object_new(&microfeed_feed_class, MicrofeedFeed);
 	feed->uri = strdup(uri);
 	feed->name = strdup(name);
+	feed->publisher = microfeed_object_ref(publisher, MicrofeedPublisher);
+	feed->object_path = strdup(microfeed_publisher_get_object_path(publisher));
 	feed->feed_permission = feed_permission;
 	feed->metadata_item = microfeed_item_new(MICROFEED_ITEM_UID_FEED_METADATA, time(NULL));
 	microfeed_item_set_property(feed->metadata_item, MICROFEED_ITEM_PROPERTY_NAME_FEED_NAME, name);
@@ -91,7 +107,6 @@ MicrofeedFeed* microfeed_feed_new(MicrofeedPublisher* publisher, const char* uri
 		microfeed_item_set_property(feed->metadata_item, MICROFEED_ITEM_PROPERTY_NAME_FEED_PERMISSION, s);
 	}
 	free(s);
-	feed->publisher = publisher;
 	feed->iterators = NULL;
 	if (callbacks) {
 		feed->callbacks = *callbacks;
@@ -103,106 +118,68 @@ MicrofeedFeed* microfeed_feed_new(MicrofeedPublisher* publisher, const char* uri
 	database_name = microfeed_util_string_concatenate(uri, ":microfeed.statuses", NULL);
 	feed->statuses_database = microfeed_database_environment_get_database(microfeed_publisher_get_database_environment(publisher), database_name, NULL);
 	free(database_name);
+	database_name = microfeed_util_string_concatenate(uri, ":microfeed.metadata", NULL);
+	feed->metadata_database = microfeed_database_environment_get_database(microfeed_publisher_get_database_environment(publisher), database_name, NULL);
+	free(database_name);
 	if (feed->callbacks.download_item_data) {
 		database_name = microfeed_util_string_concatenate(uri, ":microfeed.item_data", NULL);
 		feed->item_data_database = microfeed_database_environment_get_database(microfeed_publisher_get_database_environment(publisher), database_name, NULL);
 		free(database_name);
-		feed->get_item_data_mutexes = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)get_item_data_mutex_get_uid);
+		feed->get_item_data_mutexes = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)get_item_data_mutex_get_uid);
 	}
-	feed->mutex = microfeed_mutex_new();
+	feed->item_data_properties = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp, (MicrofeedStoreGetKeyFunction)item_data_property_get_property_name);
 
 	return feed;	
 }
 
 /* Does NOT remove subscribers */
-void microfeed_feed_free(MicrofeedFeed* feed) {
+static void free_function(MicrofeedFeed* feed) {
 	MicrofeedFeedIterator* iterator;
+	MicrofeedStoreIterator* store_iterator;
+	ItemDataProperty* item_data_property;
 
-	microfeed_weak_reference_invalidate(feed->weak_reference);
-	
 	if (feed->callbacks.destroy) {
 		feed->callbacks.destroy(feed, feed->user_data);
 	}
 	
 	free(feed->uri);
+	free(feed->name);
+	microfeed_object_unref(feed->publisher, MicrofeedPublisher);
+	free(feed->object_path);
+	microfeed_item_free(feed->metadata_item);
 	for (iterator = feed->iterators; iterator; iterator = iterator->next) {
 		iterator->feed = NULL;
 	}	
 	
-	microfeed_database_unref(feed->database);
-	microfeed_database_unref(feed->statuses_database);
+	microfeed_object_unref(feed->database, MicrofeedDatabase);
+	microfeed_object_unref(feed->statuses_database, MicrofeedDatabase);
+	microfeed_object_unref(feed->metadata_database, MicrofeedDatabase);
 	if (feed->item_data_database) {
-		microfeed_database_unref(feed->item_data_database);
+		microfeed_object_unref(feed->item_data_database, MicrofeedDatabase);
 		microfeed_store_free(feed->get_item_data_mutexes);
 	}
-	microfeed_mutex_free(feed->mutex);
+
+	for (store_iterator = microfeed_store_iterate(feed->item_data_properties, NULL);
+	     (item_data_property = microfeed_store_iterator_get(store_iterator, ItemDataProperty));
+	     microfeed_store_iterator_next(store_iterator)) {
+		free(item_data_property->property_name);
+		microfeed_object_unref(item_data_property->item_data_feed, MicrofeedFeed);
+		microfeed_memory_free(item_data_property);
+	}
+	microfeed_store_iterator_free(store_iterator);
+	microfeed_store_free(feed->item_data_properties);
 
 /* TODO: Free metadata item etc. */
 	
 	feed->uri = NULL;
-	feed->publisher = NULL;
 	feed->callbacks.destroy = NULL;
 	feed->callbacks.update = NULL;
 	feed->subscriber_count = 0;
 	feed->iterators = NULL;
-	feed->weak_reference = NULL;
-
-	free(feed);	
-}
-
-MicrofeedFeed* microfeed_feed_ref(MicrofeedFeed* feed) {
-	microfeed_mutex_lock(feed->mutex);
-
-	feed->reference_count++;
-
-	microfeed_mutex_unlock(feed->mutex);
-	
-	return feed;
-}
-	
-void microfeed_feed_unref(MicrofeedFeed* feed) {
-	microfeed_mutex_lock(feed->mutex);
-
-	feed->reference_count--;
-	
-	if (feed->reference_count == 0) {
-		microfeed_feed_free(feed);
-	} else {
-
-		microfeed_mutex_unlock(feed->mutex);
-	}
-}
-
-MicrofeedWeakReference* microfeed_feed_get_weak_reference(MicrofeedFeed* feed) {
-	MicrofeedWeakReference* weak_reference;
-	
-	microfeed_mutex_lock(feed->mutex);
-
-	weak_reference = feed->weak_reference = microfeed_weak_reference_new(feed, feed->weak_reference);
-
-	microfeed_mutex_unlock(feed->mutex);
-	
-	return weak_reference;
-}
-
-void microfeed_feed_add_free_callback(MicrofeedFeed* feed, MicrofeedFreeCallback free_callback, void* user_data) {
-	microfeed_mutex_lock(feed->mutex);
-
-	feed->weak_reference = microfeed_weak_reference_add_free_callback(feed->weak_reference, feed, free_callback, user_data);
-
-	microfeed_mutex_unlock(feed->mutex);
-}
-
-void microfeed_feed_remove_free_callback(MicrofeedFeed* feed, MicrofeedFreeCallback free_callback, void* user_data) {
-	microfeed_mutex_lock(feed->mutex);
-
-	microfeed_weak_reference_remove_free_callback(feed->weak_reference, free_callback, user_data);
-
-	microfeed_mutex_unlock(feed->mutex);
 }
 
 MicrofeedPublisher* microfeed_feed_get_publisher(MicrofeedFeed* feed) {
-	
+
 	return feed->publisher;
 }
 
@@ -224,7 +201,7 @@ void microfeed_feed_replace_item(MicrofeedFeed* feed, MicrofeedItem* item) {
 	char status;
 	MicrofeedItem* existing_item;
 
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 
 	uid = microfeed_item_get_uid(item);
 	uid_size = strlen(uid) + 1;
@@ -251,10 +228,10 @@ void microfeed_feed_replace_item(MicrofeedFeed* feed, MicrofeedItem* item) {
 				
 			existing_item = microfeed_item_new_with_status(uid, existing_timestamp, status);
 			microfeed_item_demarshal_properties(existing_item, existing_data, existing_size);
-			microfeed_publisher_handle_item_property_change(feed->publisher, existing_item, item);
+			handle_item_property_change(feed, existing_item, item);
 			microfeed_item_free(existing_item);
 			
-			microfeed_publisher_send_item_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_ITEM_CHANGED, feed->uri, item);
+			microfeed_provider_send_item_signal(microfeed_publisher_get_provider(feed->publisher), NULL, MICROFEED_SIGNAL_NAME_ITEM_CHANGED, feed->object_path, feed->uri, item);
 //			if (timestamp != existing_timestamp) {
 				microfeed_database_index_remove_data(feed->database_index, uid, uid_size);
 //			}
@@ -264,14 +241,14 @@ void microfeed_feed_replace_item(MicrofeedFeed* feed, MicrofeedItem* item) {
 		free(existing_data);
 	} else {	
 		status = (char)MICROFEED_ITEM_STATUS_NEW;
-		microfeed_publisher_handle_item_property_change(feed->publisher, NULL, item);
-		microfeed_publisher_send_item_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_ITEM_ADDED, feed->uri, item);
+		handle_item_property_change(feed, NULL, item);
+		microfeed_provider_send_item_signal(microfeed_publisher_get_provider(feed->publisher), NULL, MICROFEED_SIGNAL_NAME_ITEM_ADDED, feed->object_path, feed->uri, item);
 		microfeed_database_replace_data(feed->database, key, key_size, data, data_size);
 		microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
 	}
 	free(key);
 	
-	microfeed_mutex_unlock(feed->mutex);
+	microfeed_object_unlock(feed, MicrofeedFeed);
 }
 
 void microfeed_feed_remove_item(MicrofeedFeed* feed, const char* uid) {	
@@ -283,14 +260,14 @@ void microfeed_feed_remove_item(MicrofeedFeed* feed, const char* uid) {
 	uint64_t timestamp;
 	MicrofeedItem* item;
 
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 
 	uid_size = strlen(uid) + 1;
 	if (microfeed_database_index_get_data(feed->database_index, uid, uid_size, &key, &key_size, &data, &data_size)) {
 		timestamp = *((uint64_t*)key);
 		item = microfeed_item_new(uid, timestamp);
 		microfeed_item_demarshal_properties(item, data, data_size);
-		microfeed_publisher_handle_item_property_change(feed->publisher, item, NULL);
+		handle_item_property_change(feed, item, NULL);
 		microfeed_item_free(item);
 		free(key);
 		free(data);
@@ -300,10 +277,10 @@ void microfeed_feed_remove_item(MicrofeedFeed* feed, const char* uid) {
 		if (feed->item_data_database) {
 			microfeed_database_remove_data(feed->item_data_database, uid, uid_size);
 		}
-		microfeed_publisher_send_item_uid_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_ITEM_REMOVED, feed->uri, uid);
+		microfeed_provider_send_item_uid_signal(microfeed_publisher_get_provider(feed->publisher), NULL, MICROFEED_SIGNAL_NAME_ITEM_REMOVED, feed->object_path, feed->uri, uid);
 	}
 
-	microfeed_mutex_unlock(feed->mutex);
+	microfeed_object_unlock(feed, MicrofeedFeed);
 }
 
 void microfeed_feed_remove_items(MicrofeedFeed* feed, const char* start_uid, const char* end_uid) {
@@ -316,7 +293,7 @@ void microfeed_feed_remove_items(MicrofeedFeed* feed, const char* start_uid, con
 	uint64_t timestamp;
 	MicrofeedItem* item;
 	
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 	
 	uid_size = (start_uid ? strlen(start_uid) + 1 : 0);
 	for (iterator = microfeed_database_index_iterate(feed->database_index, start_uid, uid_size, 0);
@@ -325,7 +302,7 @@ void microfeed_feed_remove_items(MicrofeedFeed* feed, const char* start_uid, con
 		timestamp = *((uint64_t*)key);
 		item = microfeed_item_new(key + sizeof(uint64_t), timestamp);
 		microfeed_item_demarshal_properties(item, data, data_size);
-		microfeed_publisher_handle_item_property_change(feed->publisher, item, NULL);
+		handle_item_property_change(feed, item, NULL);
 		microfeed_item_free(item);
 		
 		microfeed_database_index_remove_data(feed->database_index, key + sizeof(uint64_t), key_size - sizeof(uint64_t));
@@ -333,11 +310,11 @@ void microfeed_feed_remove_items(MicrofeedFeed* feed, const char* start_uid, con
 		if (feed->item_data_database) {
 			microfeed_database_remove_data(feed->item_data_database, key + sizeof(uint64_t), key_size - sizeof(uint64_t));
 		}
-		microfeed_publisher_send_item_uid_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_ITEM_REMOVED, feed->uri, key + sizeof(uint64_t));
+		microfeed_provider_send_item_uid_signal(microfeed_publisher_get_provider(feed->publisher), NULL, MICROFEED_SIGNAL_NAME_ITEM_REMOVED, feed->object_path, feed->uri, key + sizeof(uint64_t));
 	}
 	microfeed_database_iterator_free(iterator);
 	
-	microfeed_mutex_unlock(feed->mutex);	
+	microfeed_object_unlock(feed, MicrofeedFeed);	
 }
 
 MicrofeedItem* microfeed_feed_get_item(MicrofeedFeed* feed, const char* uid) {
@@ -348,7 +325,7 @@ MicrofeedItem* microfeed_feed_get_item(MicrofeedFeed* feed, const char* uid) {
 	uint64_t timestamp;
 	MicrofeedItem* item;
 
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 	
 	if (!microfeed_database_index_get_data(feed->database_index, uid, strlen(uid) + 1, &key, &key_size, &data, &data_size)) {
 		item = NULL;
@@ -356,10 +333,11 @@ MicrofeedItem* microfeed_feed_get_item(MicrofeedFeed* feed, const char* uid) {
 		timestamp = *((uint64_t*)key);
 		item = microfeed_item_new((char*)(key + sizeof(uint64_t)), (time_t)timestamp);
 		microfeed_item_demarshal_properties(item, data, data_size);
+		free(key);
 		free(data);
 	}
 
-	microfeed_mutex_unlock(feed->mutex);
+	microfeed_object_unlock(feed, MicrofeedFeed);
 	
 	return item;	
 }
@@ -370,27 +348,51 @@ const char* microfeed_feed_get_uri(MicrofeedFeed* feed) {
 }
 
 const char* microfeed_feed_get_name(MicrofeedFeed* feed) {
-	
+
+	/* TODO: What if the name changes? */
 	return feed->name;
 }
 
 void microfeed_feed_set_name(MicrofeedFeed* feed, const char* name) {
+	microfeed_object_lock(feed, MicrofeedFeed);
+
 	if (strcmp(name, feed->name)) {
 		free(feed->name);
 		feed->name = strdup(name);
 		/* TODO: Change metadata item. */
 	}
+
+	microfeed_object_unlock(feed, MicrofeedFeed);
 }
 
 MicrofeedItem* microfeed_feed_get_metadata_item(MicrofeedFeed* feed) {
+	MicrofeedItem* item;
 
-	return feed->metadata_item;
+	microfeed_object_lock(feed, MicrofeedFeed);
+
+	item = microfeed_item_duplicate(feed->metadata_item);
+
+	microfeed_object_unlock(feed, MicrofeedFeed);
+
+	return item;
 }
 
+MicrofeedItem* microfeed_feed_send_metadata_item(MicrofeedFeed* feed, const char* destination) {
+	MicrofeedItem* item;
+
+	microfeed_object_lock(feed, MicrofeedFeed);
+
+	microfeed_provider_send_item_signal(microfeed_publisher_get_provider(feed->publisher), destination, MICROFEED_SIGNAL_NAME_ITEM_ADDED, feed->object_path, feed->uri, feed->metadata_item);
+
+	microfeed_object_unlock(feed, MicrofeedFeed);
+}
+
+
+
 MicrofeedFeedIterator* microfeed_feed_iterate(MicrofeedFeed* feed, const char* start_uid, int backwards) {
 	MicrofeedFeedIterator* iterator;
 
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 	
 	iterator = (MicrofeedFeedIterator*)microfeed_memory_allocate(MicrofeedFeedIterator);
 	iterator->feed = feed;
@@ -407,7 +409,7 @@ MicrofeedFeedIterator* microfeed_feed_iterate(MicrofeedFeed* feed, const char* s
 	iterator->previous = NULL;
 	feed->iterators = iterator;
 
-	microfeed_mutex_unlock(feed->mutex);
+	microfeed_object_unlock(feed, MicrofeedFeed);
 
 	return iterator;
 }
@@ -416,7 +418,7 @@ MicrofeedFeedIterator* microfeed_feed_iterate_timeline(MicrofeedFeed* feed, cons
 	MicrofeedFeedIterator* iterator;
 	char* key;
 
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 	
 	iterator = (MicrofeedFeedIterator*)microfeed_memory_allocate(MicrofeedFeedIterator);
 	iterator->feed = feed;
@@ -441,45 +443,47 @@ MicrofeedFeedIterator* microfeed_feed_iterate_timeline(MicrofeedFeed* feed, cons
 	iterator->previous = NULL;
 	feed->iterators = iterator;
 
-	microfeed_mutex_unlock(feed->mutex);
+	microfeed_object_unlock(feed, MicrofeedFeed);
 
 	return iterator;
 }
 
 void microfeed_feed_update(MicrofeedFeed* feed, const char* bus_name) {
 	MicrofeedError* error;
+	long timestamp;
 
-	microfeed_feed_ref(feed);
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 
 	if (feed->updating) {
 		if (bus_name) {
-			microfeed_publisher_send_feed_signal(feed->publisher, bus_name, MICROFEED_SIGNAL_NAME_FEED_UPDATE_STARTED, feed->uri);
+			microfeed_provider_send_feed_signal(microfeed_publisher_get_provider(feed->publisher), bus_name, MICROFEED_SIGNAL_NAME_FEED_UPDATE_STARTED, feed->object_path, feed->uri);
 		}
 	} else {
-		microfeed_publisher_send_feed_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_FEED_UPDATE_STARTED, feed->uri);
+		microfeed_provider_send_feed_signal(microfeed_publisher_get_provider(feed->publisher), NULL, MICROFEED_SIGNAL_NAME_FEED_UPDATE_STARTED, feed->object_path, feed->uri);
 		feed->updating = 1;
 	
 		if (feed->callbacks.update) {
 
-			microfeed_mutex_unlock(feed->mutex);
+			microfeed_object_unlock(feed, MicrofeedFeed);
 				
 			error = feed->callbacks.update(feed, (bus_name ? 1 : 0), feed->user_data);
 
-			microfeed_mutex_lock(feed->mutex);
+			microfeed_object_lock(feed, MicrofeedFeed);
 				
 			if (error) {
-				microfeed_publisher_send_error_signal(feed->publisher, NULL, microfeed_error_get_name(error), feed->uri, NULL, microfeed_error_get_message(error));
+				microfeed_provider_send_error_signal(microfeed_publisher_get_provider(feed->publisher), NULL, microfeed_error_get_name(error), feed->object_path, feed->uri, NULL, microfeed_error_get_message(error));
 				microfeed_error_free(error);
+			} else {
+				timestamp = (long)time(NULL);
+				microfeed_database_replace_data(feed->metadata_database, METADATA_LAST_UPDATED, strlen(METADATA_LAST_UPDATED) + 1, &timestamp, sizeof(long));
 			}
 		}
 
-		microfeed_publisher_send_feed_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_FEED_UPDATE_ENDED, feed->uri);
+		microfeed_provider_send_feed_signal(microfeed_publisher_get_provider(feed->publisher), NULL, MICROFEED_SIGNAL_NAME_FEED_UPDATE_ENDED, feed->object_path, feed->uri);
 		feed->updating = 0;
 	}
 
-	microfeed_mutex_unlock(feed->mutex);
-	microfeed_feed_unref(feed);
+	microfeed_object_unlock(feed, MicrofeedFeed);
 }
 
 void microfeed_feed_republish(MicrofeedFeed* feed, const char* start_uid, const char* end_uid, unsigned int max_count, const char* bus_name) {
@@ -495,9 +499,9 @@ void microfeed_feed_republish(MicrofeedFeed* feed, const char* start_uid, const
 	size_t status_data_size;
 	char status;
 	
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 	
-	microfeed_publisher_send_feed_signal(feed->publisher, bus_name, MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_STARTED, feed->uri);
+	microfeed_provider_send_feed_signal(microfeed_publisher_get_provider(feed->publisher), bus_name, MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_STARTED, feed->object_path, feed->uri);
 
 	count = 0;
 	for (iterator = microfeed_database_iterate(feed->database, end_uid, (end_uid ? strlen(end_uid) + 1 : 0), 1);
@@ -511,8 +515,9 @@ void microfeed_feed_republish(MicrofeedFeed* feed, const char* start_uid, const
 		}
 		item = microfeed_item_new_with_status(key + sizeof(uint64_t), (time_t)timestamp, (MicrofeedItemStatus)status);
 		microfeed_item_demarshal_properties(item, item_data, item_data_size);
-		microfeed_publisher_send_item_signal(feed->publisher, bus_name, MICROFEED_SIGNAL_NAME_ITEM_REPUBLISHED, feed->uri, item);
+		microfeed_provider_send_item_signal(microfeed_publisher_get_provider(feed->publisher), bus_name, MICROFEED_SIGNAL_NAME_ITEM_REPUBLISHED, feed->object_path, feed->uri, item);
 		microfeed_item_free(item);
+		free(status_data);
 	    if (start_uid && strcmp(start_uid, key + sizeof(uint64_t)) <= 0) {
 	    	break;
 	    }
@@ -522,9 +527,9 @@ void microfeed_feed_republish(MicrofeedFeed* feed, const char* start_uid, const
 	}
 	microfeed_database_iterator_free(iterator);
 
-	microfeed_publisher_send_feed_signal(feed->publisher, bus_name, MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_ENDED, feed->uri);
+	microfeed_provider_send_feed_signal(microfeed_publisher_get_provider(feed->publisher), bus_name, MICROFEED_SIGNAL_NAME_FEED_REPUBLISHING_ENDED, feed->object_path, feed->uri);
 
-	microfeed_mutex_unlock(feed->mutex);	
+	microfeed_object_unlock(feed, MicrofeedFeed);	
 }
 
 MicrofeedError* microfeed_feed_set_item_status(MicrofeedFeed* feed, const char* uid, MicrofeedItemStatus status_to_set) {
@@ -536,7 +541,7 @@ MicrofeedError* microfeed_feed_set_item_status(MicrofeedFeed* feed, const char*
 	size_t data_size;
 	char status;
 	
-	microfeed_mutex_lock(feed->mutex);	
+	microfeed_object_lock(feed, MicrofeedFeed);	
 
 	uid_size = strlen(uid) + 1;
 	if (microfeed_database_get_data(feed->statuses_database, uid, uid_size, &data, &data_size)) {
@@ -547,7 +552,7 @@ MicrofeedError* microfeed_feed_set_item_status(MicrofeedFeed* feed, const char*
 			}
 			if (!error) {
 				status |= (char)(status_to_set);
-				microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, uid, status);
+				microfeed_provider_send_status_changed_signal(microfeed_publisher_get_provider(feed->publisher), NULL, feed->object_path, feed->uri, uid, status);
 				if (status) {
 					microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
 				} else {
@@ -561,14 +566,14 @@ MicrofeedError* microfeed_feed_set_item_status(MicrofeedFeed* feed, const char*
 		}
 		if (!error) {
 			status = (char)status_to_set;
-			microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, uid, status);
+			microfeed_provider_send_status_changed_signal(microfeed_publisher_get_provider(feed->publisher), NULL, feed->object_path, feed->uri, uid, status);
 			microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
 		}
 	} else {
 		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_ITEM, "No such item when setting status.");
 	}
 
-	microfeed_mutex_unlock(feed->mutex);	
+	microfeed_object_unlock(feed, MicrofeedFeed);	
 
 	return error;
 }
@@ -580,7 +585,7 @@ MicrofeedError* microfeed_feed_unset_item_status(MicrofeedFeed* feed, const char
 	size_t data_size;
 	char status;
 	
-	microfeed_mutex_lock(feed->mutex);	
+	microfeed_object_lock(feed, MicrofeedFeed);	
 
 	uid_size = strlen(uid) + 1;
 	if (microfeed_database_get_data(feed->statuses_database, uid, uid_size, &data, &data_size)) {
@@ -591,7 +596,7 @@ MicrofeedError* microfeed_feed_unset_item_status(MicrofeedFeed* feed, const char
 			}
 			if (!error) {
 				status &= (char)(~status_to_unset);
-				microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, uid, status);
+				microfeed_provider_send_status_changed_signal(microfeed_publisher_get_provider(feed->publisher), NULL, feed->object_path, feed->uri, uid, status);
 				if (status) {
 					microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
 				} else {
@@ -603,7 +608,7 @@ MicrofeedError* microfeed_feed_unset_item_status(MicrofeedFeed* feed, const char
 		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_ITEM, "No such item when unsetting status.");
 	}
 
-	microfeed_mutex_unlock(feed->mutex);	
+	microfeed_object_unlock(feed, MicrofeedFeed);	
 
 	return error;
 }
@@ -618,7 +623,7 @@ MicrofeedError* microfeed_feed_unset_item_statuses(MicrofeedFeed* feed, const ch
 	size_t data_size;
 	char status;
 	
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 	
 	uid_size = (start_uid ? strlen(start_uid) + 1 : 0);
 	for (iterator = microfeed_database_iterate(feed->statuses_database, start_uid, uid_size, 0);
@@ -631,7 +636,7 @@ MicrofeedError* microfeed_feed_unset_item_statuses(MicrofeedFeed* feed, const ch
 			}
 			if (!error) {
 				status &= (char)(~status_to_unset);
-				microfeed_publisher_send_status_changed_signal(feed->publisher, NULL, feed->uri, (const char*)key, status);
+				microfeed_provider_send_status_changed_signal(microfeed_publisher_get_provider(feed->publisher), NULL, feed->object_path, feed->uri, (const char*)key, status);
 				if (status) {
 					microfeed_database_replace_data(feed->statuses_database, key, key_size, &status, sizeof(char));
 				} else {
@@ -642,7 +647,7 @@ MicrofeedError* microfeed_feed_unset_item_statuses(MicrofeedFeed* feed, const ch
 	}
 	microfeed_database_iterator_free(iterator);
 	
-	microfeed_mutex_unlock(feed->mutex);	
+	microfeed_object_unlock(feed, MicrofeedFeed);	
 
 	return error;
 }
@@ -651,19 +656,18 @@ MicrofeedError* microfeed_feed_send_item_data(MicrofeedFeed* feed, const char* u
 	MicrofeedError* error = NULL;
 	void* key;
 	size_t key_size;
-	void* data;
+	void* data = NULL;
 	size_t data_size;
 	GetItemDataMutex* gidm;
 	
-	microfeed_feed_ref(feed);
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 	
 	if (!feed->item_data_database) {
 		error = microfeed_error_new(MICROFEED_ERROR_NO_ITEM_DATA, "Feed does not support item data");
 	} else if (microfeed_database_get_data(feed->item_data_database, uid, strlen(uid) + 1, &data, &data_size) && data_size > sizeof(uint64_t)) {
-		microfeed_publisher_send_item_data_signal(feed->publisher, bus_name, feed->uri, uid, data + sizeof(uint64_t), data_size - sizeof(uint64_t));
+		microfeed_provider_send_item_data_signal(microfeed_publisher_get_provider(feed->publisher), bus_name, feed->object_path, feed->uri, uid, data + sizeof(uint64_t), data_size - sizeof(uint64_t));
 		free(data);
-	} else if (!microfeed_database_index_get_data(feed->database_index, uid, strlen(uid) + 1, &key, &key_size, &data, &data_size)) {
+	} else if (free(data), !microfeed_database_index_get_data(feed->database_index, uid, strlen(uid) + 1, &key, &key_size, &data, &data_size)) {
 		error = microfeed_error_new(MICROFEED_ERROR_NO_SUCH_ITEM, "Item does not exist");
 	} else {
 		free(key);
@@ -672,13 +676,13 @@ MicrofeedError* microfeed_feed_send_item_data(MicrofeedFeed* feed, const char* u
 		if ((gidm = microfeed_store_get(feed->get_item_data_mutexes, uid, GetItemDataMutex))) {
 			gidm->reference_count++;
 
-			microfeed_mutex_unlock(feed->mutex);
+			microfeed_object_unlock(feed, MicrofeedFeed);
 
 			microfeed_mutex_lock(gidm->mutex);
 
-			microfeed_mutex_lock(feed->mutex);
+			microfeed_object_lock(feed, MicrofeedFeed);
 			if (microfeed_database_get_data(feed->item_data_database, uid, strlen(uid) + 1, &data, &data_size) && data_size > sizeof(uint64_t)) {
-				microfeed_publisher_send_item_data_signal(feed->publisher, bus_name, feed->uri, uid, data + sizeof(uint64_t), data_size - sizeof(uint64_t));
+				microfeed_provider_send_item_data_signal(microfeed_publisher_get_provider(feed->publisher), bus_name, feed->object_path, feed->uri, uid, data + sizeof(uint64_t), data_size - sizeof(uint64_t));
 				free(data);
 			} else {
 				error = microfeed_error_new(MICROFEED_ERROR_NO_ITEM_DATA, "Item data does not exist");
@@ -691,14 +695,14 @@ MicrofeedError* microfeed_feed_send_item_data(MicrofeedFeed* feed, const char* u
 			microfeed_mutex_lock(gidm->mutex);
 			microfeed_store_insert(feed->get_item_data_mutexes, gidm);
 
-			microfeed_mutex_unlock(feed->mutex);
+			microfeed_object_unlock(feed, MicrofeedFeed);
 
 			error = feed->callbacks.download_item_data(feed, uid, &data, &data_size, feed->user_data);
 
-			microfeed_mutex_lock(feed->mutex);
+			microfeed_object_lock(feed, MicrofeedFeed);
 
 			if (!error) {
-				microfeed_publisher_send_item_data_signal(feed->publisher, bus_name, feed->uri, uid, data, data_size);
+				microfeed_provider_send_item_data_signal(microfeed_publisher_get_provider(feed->publisher), bus_name, feed->object_path, feed->uri, uid, data, data_size);
 
 				data = realloc(data, data_size + sizeof(uint64_t));
 				memmove(data + sizeof(uint64_t), data, data_size);
@@ -710,6 +714,7 @@ MicrofeedError* microfeed_feed_send_item_data(MicrofeedFeed* feed, const char* u
 		gidm->reference_count--;
 		if (gidm->reference_count == 0) {
 			microfeed_store_remove(feed->get_item_data_mutexes, gidm);
+			free(gidm->uid);
 			microfeed_mutex_free(gidm->mutex);
 			microfeed_memory_free(gidm);
 		} else {
@@ -717,8 +722,9 @@ MicrofeedError* microfeed_feed_send_item_data(MicrofeedFeed* feed, const char* u
 		}
 	}
 	
-	microfeed_mutex_unlock(feed->mutex);	
-	microfeed_feed_unref(feed);
+	microfeed_object_unlock(feed, MicrofeedFeed);	
+	
+	return error;
 }
 
 void microfeed_feed_ref_item_data(MicrofeedFeed* feed, const char* uid) {
@@ -733,7 +739,7 @@ void microfeed_feed_ref_item_data(MicrofeedFeed* feed, const char* uid) {
 	size_t data_size;
 	char status;
 
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 
 	if (feed->item_data_database) {
 		uid_size = strlen(uid) + 1;
@@ -751,13 +757,14 @@ void microfeed_feed_ref_item_data(MicrofeedFeed* feed, const char* uid) {
 			*((uint64_t*)key) = timestamp;
 			memcpy(key + sizeof(uint64_t), uid, uid_size);
 			microfeed_item_marshal_properties(item, &data, &data_size);
-			microfeed_publisher_send_item_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_ITEM_ADDED, feed->uri, item);
+			microfeed_provider_send_item_signal(microfeed_publisher_get_provider(feed->publisher), NULL, MICROFEED_SIGNAL_NAME_ITEM_ADDED, feed->object_path, feed->uri, item);
 			microfeed_database_replace_data(feed->database, key, key_size, data, data_size);
 			microfeed_database_replace_data(feed->statuses_database, uid, uid_size, &status, sizeof(char));
+			free(key);
 		}
 	}
 
-	microfeed_mutex_unlock(feed->mutex);
+	microfeed_object_unlock(feed, MicrofeedFeed);
 }
 
 void microfeed_feed_unref_item_data(MicrofeedFeed* feed, const char* uid) {
@@ -765,7 +772,7 @@ void microfeed_feed_unref_item_data(MicrofeedFeed* feed, const char* uid) {
 	uint64_t reference_count;
 	size_t size;
 
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 	
 	if (feed->item_data_database) {	
 		uid_size = strlen(uid) + 1;
@@ -776,14 +783,14 @@ void microfeed_feed_unref_item_data(MicrofeedFeed* feed, const char* uid) {
 				microfeed_database_index_remove_data(feed->database_index, uid, uid_size);
 				microfeed_database_remove_data(feed->statuses_database, uid, uid_size);
 				microfeed_database_remove_data(feed->item_data_database, uid, uid_size);
-				microfeed_publisher_send_item_uid_signal(feed->publisher, NULL, MICROFEED_SIGNAL_NAME_ITEM_REMOVED, feed->uri, uid);				
+				microfeed_provider_send_item_uid_signal(microfeed_publisher_get_provider(feed->publisher), NULL, MICROFEED_SIGNAL_NAME_ITEM_REMOVED, feed->object_path, feed->uri, uid);				
 			} else {
 				microfeed_database_replace_data_partial(feed->item_data_database, uid, uid_size, &reference_count, size, 0);
 			}
 		}
 	}
 	
-	microfeed_mutex_unlock(feed->mutex);
+	microfeed_object_unlock(feed, MicrofeedFeed);
 }
 
 /* TODO: Should use defines. */
@@ -815,6 +822,29 @@ char* microfeed_feed_permission_to_string(MicrofeedFeedPermission feed_permissio
 	return strdup(buffer);
 }
 
+void microfeed_feed_add_item_data_property(MicrofeedFeed* feed, const char* property_name, MicrofeedFeed* item_data_feed) {
+	ItemDataProperty* item_data_property;
+	
+	item_data_property = microfeed_memory_allocate(ItemDataProperty);
+	item_data_property->property_name = strdup(property_name);
+	item_data_property->item_data_feed = microfeed_object_ref(item_data_feed, MicrofeedFeed);
+	microfeed_store_insert(feed->item_data_properties, item_data_property);
+}
+
+long microfeed_feed_get_last_updated_time(MicrofeedFeed* feed) {
+	long last_updated_time = 0;
+	void* data;
+	size_t data_size;	
+
+	if ((microfeed_database_get_data(feed->metadata_database, METADATA_LAST_UPDATED, strlen(METADATA_LAST_UPDATED) + 1, &data, &data_size))) {
+		if (data_size == sizeof(long)) {
+			last_updated_time = *((long*)data);
+		}
+		free(data);
+	}
+	
+	return last_updated_time;
+}
 
 void microfeed_feed_iterator_free(MicrofeedFeedIterator* iterator) {
 	if (iterator->next) {
@@ -844,7 +874,7 @@ MicrofeedItem* microfeed_feed_iterator_get_item(MicrofeedFeedIterator* iterator)
 	uint64_t timestamp;
 	MicrofeedItem* item;
 
-	microfeed_mutex_lock(iterator->feed->mutex);
+	microfeed_object_lock(iterator->feed, MicrofeedFeed);
 	
 	if (!microfeed_database_iterator_get(iterator->database_iterator, &key, &key_size, &data, &data_size)) {
 		item = NULL;
@@ -854,7 +884,7 @@ MicrofeedItem* microfeed_feed_iterator_get_item(MicrofeedFeedIterator* iterator)
 		microfeed_item_demarshal_properties(item, data, data_size);
 	}
 
-	microfeed_mutex_unlock(iterator->feed->mutex);
+	microfeed_object_unlock(iterator->feed, MicrofeedFeed);
 	
 	return item;
 }
@@ -878,7 +908,7 @@ int microfeed_feed_iterator_jump_and_remove_previous_items(MicrofeedFeedIterator
 	const void* const_data;
 	size_t data_size;
 	
-	microfeed_mutex_lock(iterator->feed->mutex);
+	microfeed_object_lock(iterator->feed, MicrofeedFeed);
 	
 	uid_size = strlen(uid) + 1;
 	if (microfeed_database_index_get_data(iterator->feed->database_index, uid, uid_size, &jump_key, &jump_key_size, &data, &data_size)) {
@@ -895,10 +925,9 @@ int microfeed_feed_iterator_jump_and_remove_previous_items(MicrofeedFeedIterator
 				end_key = realloc(end_key, key_size);
 				memcpy(end_key, key, key_size);
 				end_key_size = key_size;
-				microfeed_publisher_send_item_uid_signal(iterator->feed->publisher, NULL, MICROFEED_SIGNAL_NAME_ITEM_REMOVED, iterator->feed->uri, key + sizeof(uint64_t));
+				microfeed_provider_send_item_uid_signal(microfeed_publisher_get_provider(iterator->feed->publisher), NULL, MICROFEED_SIGNAL_NAME_ITEM_REMOVED, iterator->feed->object_path, iterator->feed->uri, key + sizeof(uint64_t));
 				microfeed_database_iterator_next(iterator->database_iterator);
 			}
-			free(jump_key);
 			if (iterator->backwards) {		
 				if (iterator->timeline) {
 					microfeed_database_remove_data_range(iterator->feed->database, end_key, end_key_size, start_key, start_key_size);
@@ -917,9 +946,10 @@ int microfeed_feed_iterator_jump_and_remove_previous_items(MicrofeedFeedIterator
 			free(start_key);
 			free(end_key);
 		}
+		free(jump_key);
 	}
 	
-	microfeed_mutex_unlock(iterator->feed->mutex);
+	microfeed_object_unlock(iterator->feed, MicrofeedFeed);
 
 	return retval;
 }
@@ -927,29 +957,57 @@ int microfeed_feed_iterator_jump_and_remove_previous_items(MicrofeedFeedIterator
 MicrofeedError* microfeed_feed_call_modify_item_callback(MicrofeedFeed* feed, MicrofeedItem* existing_item, MicrofeedItem* new_item) {
 	MicrofeedError* error;
 	
-	microfeed_feed_ref(feed);
-	microfeed_mutex_lock(feed->mutex);
+	microfeed_object_lock(feed, MicrofeedFeed);
 
 /* TODO: There must NOT be on-going update. Wait somewhere until update ends... */
 
 	if (feed->callbacks.modify_item) {
 
-		microfeed_mutex_unlock(feed->mutex);
+		microfeed_object_unlock(feed, MicrofeedFeed);
 
 		error = feed->callbacks.modify_item(feed, existing_item, new_item, feed->user_data);
 
-		microfeed_mutex_lock(feed->mutex);
+		microfeed_object_lock(feed, MicrofeedFeed);
 	
 		if (error) {
-			microfeed_publisher_send_error_signal(feed->publisher, NULL, microfeed_error_get_name(error), feed->uri, microfeed_item_get_uid((existing_item ? existing_item : new_item)), microfeed_error_get_message(error));
+			microfeed_provider_send_error_signal(microfeed_publisher_get_provider(feed->publisher), NULL, microfeed_error_get_name(error), feed->object_path, feed->uri, microfeed_item_get_uid((existing_item ? existing_item : new_item)), microfeed_error_get_message(error));
 			microfeed_error_free(error);
 		}
 	} else {
 		/* TODO: Send CannotModifyItem signal */
 	}
 	
-	microfeed_mutex_unlock(feed->mutex);
-	microfeed_feed_unref(feed);
+	microfeed_object_unlock(feed, MicrofeedFeed);
+}
+
+static const char* item_data_property_get_property_name(ItemDataProperty* item_data_property) {
+
+	return item_data_property->property_name;
+}
+
+static void handle_item_property_change(MicrofeedFeed* feed, MicrofeedItem* old_item, MicrofeedItem* new_item) {
+	MicrofeedStoreIterator* iterator;
+	ItemDataProperty* item_data_property;
+	const char* old_value;
+	const char* new_value;	
+
+	for (iterator = microfeed_store_iterate(feed->item_data_properties, NULL);
+	     (item_data_property = microfeed_store_iterator_get(iterator, ItemDataProperty));
+	     microfeed_store_iterator_next(iterator)) {
+		if (old_item && (old_value = microfeed_item_get_property(old_item, item_data_property->property_name))) {
+			if (new_item && (new_value = microfeed_item_get_property(new_item, item_data_property->property_name))) {
+				if (strcmp(old_value, new_value)) {
+					microfeed_feed_unref_item_data(item_data_property->item_data_feed, old_value);
+					microfeed_feed_ref_item_data(item_data_property->item_data_feed, new_value); 
+				}
+			} else {
+				microfeed_feed_unref_item_data(item_data_property->item_data_feed, old_value);
+			}
+		} else if (new_item && (new_value = microfeed_item_get_property(new_item, item_data_property->property_name))) {
+			microfeed_feed_ref_item_data(item_data_property->item_data_feed, new_value);
+		}
+	}
+	microfeed_store_iterator_free(iterator);
 }
 
 static void index_function(void* key, const size_t key_size, void* data, const size_t data_size, void** index_key, size_t* index_key_size) {
@@ -973,9 +1031,9 @@ static void remove_old_items(MicrofeedFeed* feed) {
 	time_t timestamp;
 	int keep_old_items;
 	
-	keep_old_items = atoi(microfeed_publisher_get_setting_value(feed->publisher, "keep_old_items", "0"));
+/*	keep_old_items = atoi(microfeed_publisher_get_setting_value(feed->publisher, "keep_old_items", "0"));
 	timestamp = time(NULL) + 60 * 60 * keep_old_items;
-	
+*/	
 	/* TODO! */
 	
 }
