@@ -72,6 +72,8 @@ typedef struct {
 	char* uri;
 	char* name;
 	Subscription* subscriptions;
+	int updating : 1;
+	int republishing : 1;
 } Feed;
 
 
@@ -162,12 +164,12 @@ MicrofeedSubscriber* microfeed_subscriber_new(const char* identifier, DBusConnec
 	subscriber->configuration = microfeed_configuration_new();
 	subscriber->max_retries = 3;
 	
-	subscriber->publishers_by_identifier = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp,
-	                                                                  (MicrofeedStoreGetKeyFunction)publisher_get_identifier);
-	subscriber->providers_by_bus_name = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp,
-	                                                               (MicrofeedStoreGetKeyFunction)provider_get_bus_name);
-	subscriber->providers_by_unique_connection_name = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp,
-	                                                                             (MicrofeedStoreGetKeyFunction)provider_get_unique_connection_name);
+	subscriber->publishers_by_identifier = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp,
+	                                                                       (MicrofeedStoreGetKeyFunction)publisher_get_identifier);
+	subscriber->providers_by_bus_name = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp,
+	                                                                    (MicrofeedStoreGetKeyFunction)provider_get_bus_name);
+	subscriber->providers_by_unique_connection_name = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp,
+	                                                                                  (MicrofeedStoreGetKeyFunction)provider_get_unique_connection_name);
 
 	if (!dbus_connection_register_object_path(subscriber->connection, subscriber->object_path, &object_vtable, subscriber)) {
 	
@@ -302,7 +304,6 @@ int microfeed_subscriber_subscribe_feed(MicrofeedSubscriber* subscriber, const c
 			feed_add_subscription(feed, callbacks, callbacks_user_data);
 
 			snprintf(buffer, 1024, "type='signal',path='%s',interface='%s',arg0='%s'", publisher->object_path, MICROFEED_DBUS_INTERFACE_PUBLISHER, uri);
-			printf("%s : %d\n", buffer);
 			dbus_bus_add_match(subscriber->connection, buffer, NULL);
 			snprintf(buffer, 1024, "type='signal',path='%s',interface='%s',arg0='%s'", publisher->object_path, MICROFEED_DBUS_INTERFACE_ERROR, uri);
 			dbus_bus_add_match(subscriber->connection, buffer, NULL);
@@ -553,8 +554,8 @@ static Provider* provider_new(MicrofeedSubscriber* subscriber, const char* bus_n
 	provider = microfeed_memory_allocate(Provider);
 	provider->subscriber = subscriber;
 	provider->bus_name = strdup(bus_name);
-	provider->publishers_by_object_path = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp,
-	                                                                 (MicrofeedStoreGetKeyFunction)publisher_get_object_path);
+	provider->publishers_by_object_path = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp,
+	                                                                      (MicrofeedStoreGetKeyFunction)publisher_get_object_path);
 
 	microfeed_store_insert(subscriber->providers_by_bus_name, provider);
 	
@@ -621,8 +622,8 @@ static Publisher* publisher_new(Provider* provider, const char* identifier) {
 	} else {
 		publisher->object_path = strdup(identifier);
 	}
-	publisher->feeds = microfeed_store_new_sorted((MicrofeedStoreCompareKeysFunction)strcmp,
-		                                      (MicrofeedStoreGetKeyFunction)feed_get_uri);
+	publisher->feeds = microfeed_store_new_sorted_data((MicrofeedStoreCompareKeysFunction)strcmp,
+		                                           (MicrofeedStoreGetKeyFunction)feed_get_uri);
 	publisher->provider = provider;
 
 	microfeed_store_insert(provider->publishers_by_object_path, publisher);
@@ -758,6 +759,7 @@ static void signal_feed_update_started(MicrofeedSubscriber* subscriber, Publishe
 	Subscription* s;
 	
 	if (parse_feed_signal(publisher, message, &feed)) {
+		feed->updating = 1;
 		for (s = feed->subscriptions; s; s = s->next) {
 			if (s->callbacks->feed_update_started) {
 				s->callbacks->feed_update_started(subscriber, publisher->identifier, feed->uri, s->user_data);
@@ -771,6 +773,7 @@ static void signal_feed_update_ended(MicrofeedSubscriber* subscriber, Publisher*
 	Subscription* s;
 	
 	if (parse_feed_signal(publisher, message, &feed)) {
+		feed->updating = 0;
 		for (s = feed->subscriptions; s; s = s->next) {
 			if (s->callbacks->feed_update_ended) {
 				s->callbacks->feed_update_ended(subscriber, publisher->identifier, feed->uri, s->user_data);
@@ -784,6 +787,7 @@ static void signal_feed_republishing_started(MicrofeedSubscriber* subscriber, Pu
 	Subscription* s;
 	
 	if (parse_feed_signal(publisher, message, &feed)) {
+		feed->republishing = 1;
 		for (s = feed->subscriptions; s; s = s->next) {
 			if (s->callbacks->feed_republishing_started) {
 				s->callbacks->feed_republishing_started(subscriber, publisher->identifier, feed->uri, s->user_data);
@@ -797,6 +801,7 @@ static void signal_feed_republishing_ended(MicrofeedSubscriber* subscriber, Publ
 	Subscription* s;
 	
 	if (parse_feed_signal(publisher, message, &feed)) {
+		feed->republishing = 0;
 		for (s = feed->subscriptions; s; s = s->next) {
 			if (s->callbacks->feed_republishing_ended) {
 				s->callbacks->feed_republishing_ended(subscriber, publisher->identifier, feed->uri, s->user_data);
@@ -1046,7 +1051,8 @@ static void handle_publisher_method_return(DBusPendingCall* pending, void* user_
 		if (data->callback) {
 			data->callback(data->publisher->provider->subscriber, data->publisher->identifier, data->uri, data->uid, MICROFEED_ERROR_DBUS_MESSAGE_FAILED, "No reply from the publisher.", data->user_data);
 		}
-	} else if (dbus_message_get_type(reply) == DBUS_MESSAGE_TYPE_ERROR) {
+	} else if (dbus_message_get_type(reply) == DBUS_MESSAGE_TYPE_ERROR &&
+	           !microfeed_util_string_starts_with(dbus_message_get_error_name(reply), MICROFEED_DBUS_INTERFACE_PREFIX)) {
 		if (data->retry_counter < data->publisher->provider->subscriber->max_retries) {
 			retrying = 1;
 			if (data->retry_counter) {
@@ -1148,6 +1154,12 @@ static DBusHandlerResult object_message(DBusConnection* connection, DBusMessage*
 			     (feed = microfeed_store_iterator_get(feed_iterator, Feed));
 			     microfeed_store_iterator_next(feed_iterator)) {
 				for (s = feed->subscriptions; s; s = s->next) {
+					if (feed->republishing && s->callbacks->feed_republishing_ended) {
+						s->callbacks->feed_republishing_ended(subscriber, publisher->identifier, feed->uri, s->user_data);
+					}
+					if (feed->updating && s->callbacks->feed_update_ended) {
+						s->callbacks->feed_update_ended(subscriber, publisher->identifier, feed->uri, s->user_data);
+					}
 					if (s->callbacks->error_occured) {
 						s->callbacks->error_occured(subscriber, publisher->identifier, feed->uri, NULL, MICROFEED_ERROR_PROVIDER_CLOSED_CONNECTION, "Provider closed connection", s->user_data);	
 					}
